﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using My.Collections;
using System.Collections.Generic;


namespace WebServer
{
    public class HttpServer
    {
        TcpListener listener = null;

        public int Port { get; internal set; }
        public string Path { get; internal set; }

        public HttpServer(int port, string path)
        {
            Port = port;
            Path = path;
        }

        public void Start()
        {
            IPAddress address = Dns.GetHostEntry("localhost").AddressList[0];
            
            listener = new TcpListener(address, Port); 

            listener.Start();

            var requests = new Requests();

            while (true)
            {
                var c = listener.AcceptTcpClient();

                requests.Add(c);

                if (!listener.Pending())
                    requests.Process(Path);
            }
        }

        internal class Requests
        {
            My.Collections.Queue<TcpClient> queue = new My.Collections.Queue<TcpClient>();

            public void Add(TcpClient c)
            {
                queue.Enqueue(c);
            }

            public void Process(string path)
            {
                Console.WriteLine("Queue: " +  queue.Count);

                while (queue.Count > 0)
                {
                    try
                    {
                        var r = new Request(queue.Dequeue(), path);
                        Console.WriteLine("  " + r.LocalUrl);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine("Error: " + e.Message);
                    }
                }
            }
        }

        internal class Request
        {
            private TcpClient c;
            private string path;

            public string LocalUrl { get; private set; }
            public string Verb { get; private set; }
            public string HttpVersion { get; private set; }
            public List<string> Header { get; private set; }

            public Request(TcpClient c, string path)
            {
                this.c = c;
                this.path = path;

                Header = new List<string>();

                process();
            }

            private void process()
            {
                var r = new StreamReader(c.GetStream());    // Lesen vom Remote-Host
                var w = new StreamWriter(c.GetStream());    // Schreiben an Remote-Host

                try
                {
                    handleRequest(r, w);
                }
                catch (Exception e)
                {
                    failure(w);
                    throw e;
                }
                finally
                {
                    w.Close();
                    r.Close();
                    c.Close();
                }
            }

            private void handleRequest(StreamReader r, StreamWriter w)
            {
                string request = r.ReadLine();

                if (request != null)
                {
                    string[] tokens = request.Split(' ');

                    Verb = tokens[0].ToUpper();
                    LocalUrl = tokens[1];
                    HttpVersion = tokens[2];

                    readHeader(r);

                    string file = System.IO.Path.Combine(
                        System.IO.Path.GetFullPath(path), LocalUrl.TrimStart('/'));

                    string text = File.ReadAllText(file);

                    success(w, System.IO.Path.GetExtension(LocalUrl).TrimStart('.'));

                    w.Write(text);
                }
            }

            private void readHeader(StreamReader r)
            {
                string attribute;

                while((attribute = r.ReadLine()) != null)
                {
                    if (attribute.Equals(""))
                        break;
                    Header.Add(attribute);
                }
            }
  
            private void success(StreamWriter stream, string ext)
            {
                stream.WriteLine("HTTP/1.0 200 OK");
                stream.WriteLine("Content-Type: text/" + ext);
                stream.WriteLine("Connection: close");
                stream.WriteLine("");
            }

            private void failure(StreamWriter stream)
            {
                stream.WriteLine("HTTP/1.0 404 not found");
                stream.WriteLine("Connection: close");
                stream.WriteLine("");
            }
        }
    }
}
